@@ -39,6 +39,12 @@ syscall_bpf (int cmd, union bpf_attr *attr, unsigned int size)
 
 #endif
 
+static inline uint64_t
+ptr_to_u64 (const void *ptr)
+{
+  return (uint64_t) (unsigned long) ptr;
+}
+
 enum
 {
   HAS_WILDCARD = 1
@@ -70,6 +76,9 @@ struct bpf_program
 #  define BPF_JMP_IMM(OP, DST, IMM, OFF) \
     ((struct bpf_insn){ .code = BPF_JMP | BPF_OP (OP) | BPF_K, .dst_reg = DST, .src_reg = 0, .off = OFF, .imm = IMM })
 
+#  define BPF_JMP_REG(OP, DST, SRC, OFF) \
+    ((struct bpf_insn){ .code = BPF_JMP | BPF_OP (OP) | BPF_X, .dst_reg = DST, .src_reg = SRC, .off = OFF, .imm = 0 })
+
 #  define BPF_MOV64_IMM(DST, IMM) \
     ((struct bpf_insn){ .code = BPF_ALU64 | BPF_MOV | BPF_K, .dst_reg = DST, .src_reg = 0, .off = 0, .imm = IMM })
 
@@ -156,8 +165,11 @@ bpf_program_append_dev (struct bpf_program *program, const char *access, char ty
     BPF_EXIT_INSN (),
   };
 
+  if (access == NULL)
+    access = "";
+
   if (program->private & HAS_WILDCARD)
-    return 0;
+    return program;
 
   for (i = 0; access[i]; i++)
     {
@@ -180,7 +192,7 @@ bpf_program_append_dev (struct bpf_program *program, const char *access, char ty
   /*
     if (request.type != device.type)
       goto next_block:
-    if ((request.access & device.access) == 0)
+    if ((request.access & device.access) != request.access)
       goto next_block:
     if (device.major != '*' && request.major != device.major) == 0)
       goto next_block:
@@ -207,7 +219,7 @@ bpf_program_append_dev (struct bpf_program *program, const char *access, char ty
       struct bpf_insn i[] = {
         BPF_MOV32_REG (BPF_REG_1, BPF_REG_3),
         BPF_ALU32_IMM (BPF_AND, BPF_REG_1, bpf_access),
-        BPF_JMP_IMM (BPF_JEQ, BPF_REG_1, 0, number_instructions - 2),
+        BPF_JMP_REG (BPF_JNE, BPF_REG_1, BPF_REG_3, number_instructions - 2),
       };
       number_instructions -= 3;
       program = bpf_program_append (program, i, sizeof (i));
@@ -275,7 +287,7 @@ read_all_progs (int dirfd, uint32_t **progs_out, size_t *n_progs_out, libcrun_er
       attr.query.target_fd = dirfd;
       attr.query.attach_type = BPF_CGROUP_DEVICE;
       attr.query.prog_cnt = cur_size;
-      attr.query.prog_ids = (uint64_t) progs;
+      attr.query.prog_ids = ptr_to_u64 (progs);
 
       ret = bpf (BPF_PROG_QUERY, &attr, sizeof (attr));
     }
@@ -459,9 +471,9 @@ libcrun_ebpf_load (struct bpf_program *program, int dirfd, const char *pin, libc
 
   memset (&attr, 0, sizeof (attr));
   attr.prog_type = BPF_PROG_TYPE_CGROUP_DEVICE;
-  attr.insns = (uint64_t) program->program;
+  attr.insns = ptr_to_u64 (program->program);
   attr.insn_cnt = bpf_program_instructions (program);
-  attr.license = (uint64_t) "GPL";
+  attr.license = ptr_to_u64 ("GPL");
 
   /* First try without log.  */
   fd = bpf (BPF_PROG_LOAD, &attr, sizeof (attr));
@@ -472,7 +484,7 @@ libcrun_ebpf_load (struct bpf_program *program, int dirfd, const char *pin, libc
 
       log[0] = '\0';
       attr.log_level = 1;
-      attr.log_buf = (uint64_t) log;
+      attr.log_buf = ptr_to_u64 (log);
       attr.log_size = log_size;
 
       fd = bpf (BPF_PROG_LOAD, &attr, sizeof (attr));
@@ -490,7 +502,7 @@ libcrun_ebpf_load (struct bpf_program *program, int dirfd, const char *pin, libc
       unlink (pin);
 
       memset (&attr, 0, sizeof (attr));
-      attr.pathname = (uint64_t) pin;
+      attr.pathname = ptr_to_u64 (pin);
       attr.bpf_fd = fd;
       ret = bpf (BPF_OBJ_PIN, &attr, sizeof (attr));
       if (ret < 0)
