@@ -4,7 +4,7 @@
  * Copyright (C) 2017, 2018, 2019 Giuseppe Scrivano <giuseppe@scrivano.org>
  * crun is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * crun is distributed in the hope that it will be useful,
@@ -36,9 +36,8 @@ struct terminal_status_s
   struct termios termios;
 };
 
-
 int
-libcrun_new_terminal (char **slave, libcrun_error_t *err)
+libcrun_new_terminal (char **pty, libcrun_error_t *err)
 {
   char buf[64];
   int ret;
@@ -54,7 +53,7 @@ libcrun_new_terminal (char **slave, libcrun_error_t *err)
   if (UNLIKELY (ret < 0))
     return crun_make_error (err, errno, "unlockpt");
 
-  *slave = xstrdup (buf);
+  *pty = xstrdup (buf);
 
   ret = fd;
   fd = -1;
@@ -63,7 +62,7 @@ libcrun_new_terminal (char **slave, libcrun_error_t *err)
 }
 
 static int
-set_raw (int fd, libcrun_error_t *err)
+set_raw (int fd, void **current_status, libcrun_error_t *err)
 {
   int ret;
   struct termios termios;
@@ -72,6 +71,14 @@ set_raw (int fd, libcrun_error_t *err)
   if (UNLIKELY (ret < 0))
     return crun_make_error (err, errno, "tcgetattr");
 
+  if (current_status)
+    {
+      struct terminal_status_s *s = xmalloc (sizeof (*s));
+      s->fd = fd;
+      memcpy (&(s->termios), &termios, sizeof (termios));
+      *current_status = s;
+    }
+
   cfmakeraw (&termios);
 
   termios.c_iflag &= OPOST;
@@ -85,13 +92,13 @@ set_raw (int fd, libcrun_error_t *err)
 }
 
 int
-libcrun_set_stdio (char *slave, libcrun_error_t *err)
+libcrun_set_stdio (char *pty, libcrun_error_t *err)
 {
   int ret, i;
-  cleanup_close int fd = open (slave, O_RDWR);
+  cleanup_close int fd = open (pty, O_RDWR);
 
   if (UNLIKELY (fd < 0))
-    return crun_make_error (err, errno, "open %s", slave);
+    return crun_make_error (err, errno, "open %s", pty);
 
   for (i = 0; i < 3; i++)
     {
@@ -108,7 +115,7 @@ libcrun_set_stdio (char *slave, libcrun_error_t *err)
 }
 
 int
-libcrun_setup_terminal_master (int fd, void **current_status, libcrun_error_t *err)
+libcrun_setup_terminal_ptmx (int fd, void **current_status, libcrun_error_t *err)
 {
   int ret;
   struct termios termios;
@@ -117,19 +124,11 @@ libcrun_setup_terminal_master (int fd, void **current_status, libcrun_error_t *e
   if (UNLIKELY (ret < 0))
     return crun_make_error (err, errno, "tcgetattr");
 
-  if (current_status)
-    {
-      struct terminal_status_s *s = xmalloc (sizeof (*s));
-      s->fd = 0;
-      memcpy (&(s->termios), &termios, sizeof (termios));
-      *current_status = s;
-    }
-
   ret = tcsetattr (fd, TCSANOW, &termios);
   if (UNLIKELY (ret < 0))
     return crun_make_error (err, errno, "tcsetattr");
 
-  return set_raw (0, err);
+  return set_raw (0, current_status, err);
 }
 
 void
@@ -146,7 +145,7 @@ cleanup_terminalp (void *p)
 int
 libcrun_terminal_setup_size (int fd, unsigned short rows, unsigned short cols, libcrun_error_t *err)
 {
-  struct winsize ws = {.ws_row = rows, .ws_col = cols};
+  struct winsize ws = { .ws_row = rows, .ws_col = cols };
   int ret;
 
   if (ws.ws_row == 0 || ws.ws_col == 0)
